#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------


class BookFile(object):
	def __init__(self, fname, fcontent, fmime):
		self.fname = fname
		self.fcontent = fcontent
		self.mimetype = fmime

	def enqueue(self, stash):
		stash.data_queue.append(self)

class BookFileText(BookFile):
	def __init__(self, fname, fcontent):
		self.folder = "Text"
		super(BookFileText, self).__init__(fname, fcontent.encode("utf8"), "application/xhtml+xml")

	def enqueue(self, stash):
		stash.text_queue.append(self)
		
class BookFileStyle(BookFile):
	def __init__(self, fname, fcontent):
		self.folder = "Styles"
		super(BookFileStyle, self).__init__(fname, fcontent, "text/css")

class BookFileImage(BookFile):
	def __init__(self, fname, fcontent, fmime):
		self.folder = "Images"
		super(BookFileImage, self).__init__(fname, fcontent, fmime)

class BookFileFont(BookFile):
	def __init__(self, fname, fcontent, fmime):
		self.folder = "Fonts"
		super(BookFileFont, self).__init__(fname, fcontent, fmime)

class BookFileUnknown(BookFile):
	def __init__(self, fname, fcontent, folder, fmime):
		self.folder = folder
		super(BookFileUnknown, self).__init__(fname, fcontent, fmime)



#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------

# This GDrive class is based on a sample file provided by Google

import logging
import uuid

import apiclient
import markdown

import sanitizer
import bookfile

log = logging.getLogger(__name__)

# TODO: Implement this
class FileStorage():
	pass

class GDriveFileProcessorFactory(FileStorage):
	def get_processor(self, dfile):
		font_mime_type_list = [
			"application/font-woff",
			"application/font-otf",
			"application/x-font-ttf",
			"application/x-font-opentype",
			"application/font-sfnt"
		]
		mime_type = dfile['mimeType']

		log.debug("File name: %s, mime type: %s" % (dfile.get('title'), mime_type))

		if mime_type == "application/vnd.google-apps.document":
			return GDriveDocProcessor()
		elif mime_type == "text/x-markdown":
			return GDriveMDProcessor()
		elif mime_type.startswith("image/"):
			return GDriveImageProcessor()
		elif mime_type == "text/css":
			return GDriveStyleProcessor()
		elif mime_type in font_mime_type_list:
			return GDriveFontProcessor()
		elif mime_type in ["application/xml", "text/xml"]:
			return GDriveFileProcessor()
		elif mime_type == "application/vnd.google-apps.spreadsheet":
			return GDriveSpreadProcessor()
		elif mime_type in ["application/xhtml+xml", "text/html"]:
			return GDriveXHTMLProcessor()
		else:
			log.warning("Unknown mime type: %s (file: %s)" % (mime_type, dfile.get('title')))
			return GDriveFileProcessor()

def GDriveDownloader(service, download_url):
	if download_url:
		resp, content = service._http.request(download_url)
		if resp.status == 200:
			log.debug("Downloader: Status: %s" % resp)
			return content
		else:
			log.error("Downloader: An error occurred: %s" % resp)
			return None
	else:
		# The file doesn't have any content stored on Drive.
		return None


class GDriveFileProcessor(object):
	def process(self, service, dfile):
		content = self.download(service, dfile)
		filename = dfile.get('title').replace(" ", "")
		return bookfile.BookFileUnknown(filename, content, folder=None, fmime=dfile['mimeType'])

	def download(self, service, dfile):
		download_url = dfile.get('downloadUrl')
		return GDriveDownloader(service, download_url)

class GDriveImageProcessor(GDriveFileProcessor):
	def process(self, service, dfile):
		content = self.download(service, dfile)
		filename = dfile.get('title').replace(" ", "")
		return bookfile.BookFileImage(filename, content, dfile['mimeType'])

class GDriveFontProcessor(GDriveFileProcessor):
	def process(self, service, dfile):
		content = self.download(service, dfile)
		filename = dfile.get('title').replace(" ", "")
		return bookfile.BookFileFont(filename, content, dfile['mimeType'])

class GDriveStyleProcessor(GDriveFileProcessor):
	def process(self, service, dfile):
		content = self.download(service, dfile)
		filename = dfile.get('title').replace(" ", "")
		return bookfile.BookFileStyle(filename, content)

class GDriveXHTMLProcessor(GDriveFileProcessor):
	def process(self, service, dfile):
		content = self.download(service, dfile)
		filename = dfile.get('title').replace(" ", "")
		return bookfile.BookFileUnknown(filename, content, folder="Text", fmime="application/xhtml+xml")

class GDriveDocProcessor(GDriveFileProcessor):
	def process(self, service, dfile):
		html_content = self.download(service, dfile)
		sanitzr = sanitizer.Sanitizer()
		html_content = sanitzr.sanitize(html_content)
		filename = "text__" + dfile.get('title').replace(" ", "") + "_" + unicode(uuid.uuid4()) + ".html"

		return bookfile.BookFileText(filename, html_content)

	def download(self, service, dfile):
		download_url = dfile['exportLinks']['text/html']
		return GDriveDownloader(service, download_url)

class GDriveMDProcessor(GDriveFileProcessor):
	def process(self, service, dfile):
		md_content = self.download(service, dfile)
		xml_content = markdown.markdown(md_content.decode("utf8"), extensions=['extra'])
		html_content = sanitizer.wrap_content(xml_content)
		filename = "text__" + dfile.get('title').replace(" ", "") + "_" + unicode(uuid.uuid4()) + ".html"
		return bookfile.BookFileText(filename, html_content)

class GDriveSpreadProcessor(GDriveFileProcessor):
	def process(self, service, dfile):
		csv_content = self.download(service, dfile)	
		filename = dfile.get('title')
		return bookfile.BookFileUnknown(filename, csv_content, folder=None, fmime=None)

	def download(self, service, dfile):
		pdf_url = dfile['exportLinks']['application/pdf']
		csv_url = pdf_url[:len(pdf_url)-3] + "csv"
		return GDriveDownloader(service, csv_url)


class GDrive(FileStorage):
	def __init__(self, cache=False):
		self.cache = cache
		self.authorize()
	
	def authorize(self):
		import httplib2
		import pprint

		from apiclient.discovery import build
		from oauth2client.client import OAuth2WebServerFlow
		from oauth2client.file import Storage

		# Copy your credentials from the APIs Console (https://code.google.com/apis/console/)
		CLIENT_ID = '447065192799-5uofsvr635rm62fpt5nncj69g60rglf1.apps.googleusercontent.com'
		CLIENT_SECRET = '9KoSX3kpNPFUY4TV4rXZgZjD'

		# Check https://developers.google.com/drive/scopes for all available scopes
		OAUTH_SCOPE = 'https://www.googleapis.com/auth/drive'

		# Redirect URI for installed apps
		REDIRECT_URI = 'urn:ietf:wg:oauth:2.0:oob'
		#REDIRECT_URI = 'http://localhost:8000'

		authorized = False
		if self.cache:
			# Path to the crendentials
			CRED_FILENAME = 'credentials'

			### For storing token
			storage = Storage(CRED_FILENAME)

			if storage.get():
				credentials = storage.get()
				authorized = True

		if (not self.cache) or (not authorized):
			# Run through the OAuth flow and retrieve authorization code
			flow = OAuth2WebServerFlow(CLIENT_ID, CLIENT_SECRET, OAUTH_SCOPE, REDIRECT_URI)
			authorize_url = flow.step1_get_authorize_url()
			print "=========AUTHORIZATION=============="
			print 'Go to the following link in your browser:\n' + authorize_url
			print "===================================="
			code = raw_input('Enter verification code: ').strip()
			credentials = flow.step2_exchange(code)

			### Storing access token and a refresh token in CRED_FILENAME
			if self.cache:
				storage.put(credentials)

		# Create an httplib2.Http object and authorize it with our credentials
		http = httplib2.Http()
		http = credentials.authorize(http)

		self.service = build('drive', 'v2', http=http)

	# TODO: Implement
	def verify_folder(self, service, folder_id):
		# Does the folder exist?
		pass


	def retrieve_all_files(self):
		from apiclient import errors
		result = []
		page_token = None
		while True:
			try:
				param = {}
				if page_token:
					param['pageToken'] = page_token
				files = self.service.files().list(**param).execute()

				result.extend(files['items'])
				page_token = files.get('nextPageToken')
				if not page_token:
					break
			except errors.HttpError, error:
				log.error("An error occurred: %s" % error)
				break
		return result

	def get_file_list(self, folder_id):
		from apiclient import errors
		file_ids = []
		page_token = None
		while True:
			try:
				param = {"q": "trashed=false and mimeType != 'application/vnd.google-apps.folder'"}
				if page_token:
					param['pageToken'] = page_token

				children = self.service.children().list(
								folderId=folder_id, **param).execute()

				for child in children.get('items', []):
					file_ids.append(child['id'])

				page_token = children.get('nextPageToken')

				if not page_token:
					break

			except errors.HttpError, error:
				log.error("An error occurred: %s" % error)
				break
		
		return file_ids

	def get_file_metadata(self, file_id):
		return self.service.files().get(fileId=file_id).execute()


	def print_file_metadata(self, log, file_id, prefix="\t"):
		from apiclient import errors
		try:
			_file = self.get_file_metadata(file_id)

			log.info("{}Title: {}".format(prefix, _file["title"].encode("utf8")))
			log.info("{}MIME type:: {}".format(prefix, _file["mimeType"].encode("utf8")))
		except errors.HttpError, error:
			log.error("An error occurred: %s" % error)


if __name__ == '__main__':
	drive = GDrive()
	file_ids = drive.get_file_list("0B--lHppejR64bXBJZ3YwT0tJM00")

	print 'Number of files: %s' % len(file_ids)
	
	for id in file_ids:
		drive.download_file(id)




# -*- coding: utf-8 -*-

"""
Copyright (c) 2012, Bin Tan
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------


# This source file is based on Bin Tan's python-epub-builder
# located at https://code.google.com/p/python-epub-builder/
# The code is heavily modified

import itertools
import os
import shutil
import subprocess
import uuid
import zipfile
import re
from genshi.template import TemplateLoader
import logging

import templates

log = logging.getLogger(__name__)

class EpubItem:
	def __init__(self):
		self.id = u""
		self.path = u""
		self.mime_type = u""

class EpubBook:
	def __init__(self):
		self.loader = TemplateLoader(
								os.path.join(os.path.dirname(__file__), 'templates'),
								auto_reload=True
								)

		self.rootDir = u""
		self.uuid = uuid.uuid1()

		self.title = u""
		self.meta_info = []

		self.items = []

		self.cover_image = None
		self.title_page = None

		self.spine = []
		self.guide = {}

		self.toc_root = None

	def set_title(self, title):
		self.title = title
		
	def add_meta(self, metaName, metaValue, **metaAttrs):
		self.meta_info.append((unicode(metaName), unicode(metaValue), metaAttrs))
	
	def get_meta_tags(self):
		l = []
		for metaName, metaValue, metaAttr in self.meta_info:
			beginTag = '<dc:%s' % metaName
			if metaAttr:
				for attrName, attrValue in metaAttr.iteritems():
					beginTag += ' opf:%s="%s"' % (attrName, attrValue)
			beginTag += '>'
			endTag = '</dc:%s>' % metaName
			l.append((beginTag, metaValue, endTag))
		return l

	def get_all_items(self):
		return sorted(self.items, key = lambda x : x.id)

	def add_item(self, bookfile):
		item = EpubItem()
		item.id = re.sub(r'\W+', '_', unicode(bookfile.folder + "_" + bookfile.fname))
		item.path = unicode(bookfile.folder + "/" + bookfile.fname)
		item.mime_type = unicode(bookfile.mimetype)
		self.items.append(item)
		return item

	def set_toc_root(self, root):
		self.toc_root = root

	def get_spine(self):
		return sorted(self.spine)
	
	def add_spine_item(self, item, linear = True, order = None):
		if order == None:
			order = (max(order for order, _, _ in self.spine) if self.spine else 0) + 1
		self.spine.append((order, item, linear))

	def get_guide(self):
		return sorted(self.guide.values(), key = lambda x : x[2])

	def add_guide_item(self, href, title, type):
		assert type not in self.guide
		self.guide[type] = (href, title, type)

	def __write_container_XML(self):
		fout = open(os.path.join(self.rootDir, 'META-INF', 'container.xml'), 'w')
		fout.write(templates.container_xml)
		fout.close()

	def __write_toc_NCX(self):
		fout = open(os.path.join(self.rootDir, 'OEBPS', 'toc.ncx'), 'w')
		toc = templates.toc_ncx.format(book=self, navmap=self.toc_root.export_navmap())
		fout.write(toc.encode("utf8"))
		fout.close()
	
	def __write_content_OPF(self):
		fout = open(os.path.join(self.rootDir, 'OEBPS', 'content.opf'), 'w')
		tmpl = self.loader.load('content.opf')
		stream = tmpl.generate(book = self)
		fout.write(stream.render('xml').encode("utf8"))
		fout.close()

	def __write_mimetype(self):
		fout = open(os.path.join(self.rootDir, 'mimetype'), 'w')
		fout.write('application/epub+zip')
		fout.close()

	@staticmethod
	def __listManifestItems(contentOPFPath):
		from bs4 import BeautifulSoup
		
		items = []
		with open(contentOPFPath, "r") as f:
			soup = BeautifulSoup(f)
			for item in soup.find_all("opf:item"):
				items.append(item["href"])
		return items

	@staticmethod
	def create_archive(input_path, output_path):
		fout = zipfile.ZipFile(output_path, 'w')
		cwd = os.getcwd()
		os.chdir(input_path)
		fout.write('mimetype', compress_type = zipfile.ZIP_STORED)
		fileList = []
		fileList.append(os.path.join('META-INF', 'container.xml'))
		fileList.append(os.path.join('OEBPS', 'content.opf'))
		for itemPath in EpubBook.__listManifestItems(os.path.join('OEBPS', 'content.opf')):
				fileList.append(os.path.join('OEBPS', itemPath))

		for filePath in fileList:
			fout.write(filePath, compress_type = zipfile.ZIP_DEFLATED)
		fout.close()
		os.chdir(cwd)
	
	def create_book(self, rootDir):
		self.rootDir = rootDir
		self.__write_mimetype()
		self.__write_container_XML()
		self.__write_content_OPF()
		self.__write_toc_NCX()



#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------

import logging

log = logging.getLogger(__name__)

class TocNode(object):
	def __init__(self, level, playorder, text, source, empty=False):
		self.playorder = playorder
		self.text = text
		self.source = source
		self.level = level
		self.children = []
		self.parent = None
		self.maxlevel = 0
		self.empty = empty
		
	def add_child(self, child):
		if child.level == self.level + 1:
			self.children.append(child)
			child.parent = self
			
		elif len(self.children) > 0:
			level = self.children[-1].add_child(child)
			
		else:
			log.warning("There is a jump in document heading structure: {}".format(child.text.encode("utf8")))
			empty_child = TocNode(self.level + 1, self.playorder + 1, "", child.source, empty=True)
			self.children.append(empty_child)
			empty_child.add_child(child)
			
		if child.level > self.maxlevel:
			self.maxlevel = child.level

	def __str__(self):
		print "\t"*self.level + self.text
		for child in self.children:
			child.__str__()

	def export_navmap(self):
		navmap = u"<navMap>\n"
		
		for child in self.children:
			navmap = navmap + child.export_navpoint()
			
		navmap = navmap + u"</navMap>"
		
		return navmap

	def export_navpoint(self):
		navpoint = u""
		first_indent = u"  "*self.level
		second_indent = first_indent + "  "
		if self.empty == False:
			navpoint = first_indent + u'<navPoint id="navPoint-{}" playOrder="{}">\n'.format(self.playorder, self.playorder)
			navpoint = navpoint + second_indent + u'<navLabel><text>{}</text></navLabel>\n'.format(self.text)
			navpoint = navpoint + second_indent + u'<content src="{}"/>\n'.format(self.source)
		if len(self.children) > 0:
			for child in self.children:
				navpoint = navpoint + child.export_navpoint()
		if self.empty == False:
			navpoint = navpoint + first_indent + u'</navPoint>\n'
		
		return navpoint




#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------


import os
import tempfile
import itertools
import logging
import re
import uuid   #needed for embedding images
import urllib #needed for embedding images

from bs4 import BeautifulSoup

import metadata
import epub
import toc
import bookfile  #needed for embedding images


log = logging.getLogger(__name__)

class Stash(object):
	def __init__(self, base, epub_name):
		self.base = base
		self.epub_name = epub_name

		self.text_queue = []
		self.data_queue = []

		self.process_queue = []
		self.content_queue = []

		# Lists is only used for printing debug info
		self.lists = {
				"Text": self.text_queue,
				"Data": self.data_queue
			}

		self.metadata = metadata.Metadata()

	def mkdirs(self):
		import os, errno

		dir_list = ["META-INF", "OEBPS", "OEBPS/Fonts", "OEBPS/Images", "OEBPS/Styles", "OEBPS/Text"]

		for d in dir_list:
			path =  os.path.join(self.base, d)
			log.debug("Creating directory: %s" % path)

			try:
				os.makedirs(path)
			except OSError as exc:
				if exc.errno == errno.EEXIST and os.path.isdir(path):
					pass
				else:
					raise

	#TODO: Does this really belong here?
	def mark_footnotes(self, bf):
		soup = BeautifulSoup(bf.fcontent)
		body = soup.body

		log.debug("Looking for footnotes: {}".format(bf.fname.encode("utf8")))

		for div in body.find_all("div"):
			for a in div.find_all("a"):
				if a.get("href", "").startswith("#ftnt_"):
					log.debug("Found footnote.")
					div['class'] = div.get("class", []) + ["footnote"]

		soup.body.replace_with(body)
		return str(soup)

	#TODO: Does this really belong here?
	def embed_images(self, bf):
		soup = BeautifulSoup(bf.fcontent)
		body = soup.body

		log.debug("Looking for images: {}".format(bf.fname.encode("utf8")))
		for img in body.find_all("img"):
			log.debug("Image found: {}".format(img.get("alt", "").encode("utf8")))
			log.debug("Image source: {}".format(img.get("src", "").encode("utf8")))
			img_name, img_extension = os.path.splitext(img.get("alt", ""))
			img_name = str(uuid.uuid4()) + img_extension
			log.debug("New image name: {}".format(img_name.encode("utf8")))
			img_content = urllib.urlopen(img.get("src", "")).read()

			img_fmime = ""

			if img_extension.lower() in [".png"]:
				img_fmime = "image/png"
			elif img_extension.lower() in [".gif"]:
				img_fmime = "image/gif"
			elif img_extension.lower() in [".jpg", ".jpeg"]:
				img_fmime = "image/jpeg"
			else:
				log.debug("Unknown file type: {}".format(img_extension.encode("utf8")))
				return

			img_file = bookfile.BookFileImage(img_name, img_content, img_fmime)

			img_file.enqueue(self)

			img["src"] = "../Images/" + img_name
			del img["style"]
			del img["title"]

		soup.body.replace_with(body)
		return str(soup)

	def prepare_content_files(self):
		"""
		Sort and rename the content files
		"""
		log.info("Preparing content files...")
		self.text_queue = sorted(self.text_queue, key = lambda x : x.fname)

		counter = 1
		for bf in self.text_queue:
			bf.fcontent = self.embed_images(bf)
			bf.fcontent = self.mark_footnotes(bf)
			log.debug("Renaming {}".format(bf.fname.encode("utf8")))
			bf.fname = "Section%04d.xhtml" % counter
			counter = counter + 1

	def store_files_to_disk(self):
		"""
		Store content files to disk and add all files to
		either the content queue or the process queue
		for further processing.
		"""
		for bf in itertools.chain(self.text_queue, self.data_queue):
			if bf.folder == None:
				# Not a typical content file, needs further processing
				log.debug("Data file: %s" % bf.fname)
				self.process_queue.append(bf)
			else:
				filepath =  os.path.join(self.base, "OEBPS", bf.folder, bf.fname)
				log.debug("Exporting: %s" % filepath)

				with open(filepath, "w") as f:
					f.write(bf.fcontent)
				self.content_queue.append(bf)

		self.content_queue = sorted(self.content_queue, key=lambda bf : bf.fname.lower())

	def process_remaining_files(self):
		"""
		Process the non-content files (e.g., metadata)
		"""
		log.info("Processing data files...")
		for f in self.process_queue:
			if f.fname == "metadata":
				log.info("Processing metadata...")
				with open("temp.csv", "w") as tf:
					tf.write(f.fcontent)
				self.metadata.extract_from_csv("temp.csv")
				os.remove("temp.csv")
			else:
				log.warning("File not processed: %s, mime type: %s" % (f.fname, f.mimetype))

	def update_book_metadata(self, book):
		book.set_title(self.metadata.first_title)

		for name, value, role, file_as, event, scheme in self.metadata.meta_list:
			kwargs = {}
			if role != "":
				kwargs["role"] = role
			if file_as != "":
				kwargs["file-as"] = file_as
			if event != "":
				kwargs["event"] = event
			if scheme != "":
				kwargs["scheme"] = scheme

			book.add_meta(name, value, **kwargs)
			
	def append_head_tags(self, soup, style_list):
		log.debug("Generating the stylesheet link tags...")
		styles = sorted(style_list, key = lambda x : x.id)

		style_tag = ""
		for item in styles:
			style_tag = style_tag + """<link href="../%s" rel="stylesheet" type="text/css"/>\n""" % item.path

		links = BeautifulSoup(style_tag)
		
		log.debug("Generating the new <title> tag...")
		new_title = soup.new_tag("title")
		new_title.string = self.metadata.first_title
		
		soup.head.append(links)
		if soup.title:
			soup.title.replace_with(new_title)
		else:
			soup.head.append(new_title)
		
		return soup
			
	def update_html_tags(self, toc_list, style_list):
		log.debug("Going through the content files for tag update...")
		
		for item in toc_list:
			#TODO: Refactor
			with open(os.path.join(self.base, "OEBPS", item.path), "r") as f:
				soup = BeautifulSoup(f)

				if soup.html:
					if soup.head == None:
						log.warning("No <head> tag in HTML, creating one. File: %s" % item.path)
						head_tag = soup.new_tag("head")
						soup.html.insert(0, head_tag)
					
					log.debug("Linking stylesheet(s) and updating <title> tags. File: %s" % item.path)
					soup = self.append_head_tags(soup, style_list)
				else:
					log.error("HTML file doesn't contain the <html> tag. Stylesheet(s) cannot be added. File: %s" % item.path)
					
			with open(os.path.join(self.base, "OEBPS", item.path), "w") as f:
				f.write(unicode(soup).encode("utf8"))

	
	def generate_toc(self, book, toc_list):
		root = toc.TocNode(0, 0, "", "")
		
		log.info("Generating table of contents...")
		playorder = 1
		for item in toc_list:
			path = os.path.join(self.base, "OEBPS", item.path)
			source_path = re.compile('^.*/OEBPS/').sub('', path)
			with open(path, "r") as f:
				soup = BeautifulSoup(f)
				for header in soup.findAll(['h1', 'h2', 'h3', 'h4', 'h5', 'h6']):
					depth = int(float(header.name[1:]))
					log.debug("Heading found (depth: {}): {}".format(str(depth), header))
					try:
						source = source_path + "#" + header["id"].decode("utf8")
					except:
						log.debug("Heading doesn't have an ID. Creating one...")
						header["id"] = u"bbheading-" + unicode(playorder)
						source = source_path + "#" + header["id"]
						log.debug("New heading: {}".format(header.encode("utf8")))

					text = header.text
					node = toc.TocNode(depth, playorder, text, source)
					root.add_child(node)
					playorder = playorder + 1

			#TODO opening the file twice
			with open(path, "w") as f:
				f.write(unicode(soup).encode("utf8"))
		
		log.debug("Table of contents generation complete...")
		log.debug("TOC depth: {}".format(root.maxlevel))
		book.set_toc_root(root)

	def generate_epub(self):
		import shutil

		book = epub.EpubBook()

		self.update_book_metadata(book)

		toc_list = []
		style_list = []

		for bf in self.content_queue:
			item = book.add_item(bf)
			
			if item.path.startswith("Text"):
				book.add_spine_item(item)
				toc_list.append(item)
			elif item.path.startswith("Styles"):
				style_list.append(item)
				
		self.update_html_tags(toc_list, style_list)

		self.generate_toc(book, toc_list)

		book.create_book(self.base)
		epub_file_path = os.path.join(os.getcwd(), self.epub_name + ".epub")
		epub.EpubBook.create_archive(self.base, epub_file_path)

		shutil.rmtree(self.base)

		return epub_file_path

	def export(self):
		log.debug("Exporting stash...")
		self.mkdirs()
		self.prepare_content_files()
		self.store_files_to_disk()
		self.process_remaining_files()

		epub_file_path = self.generate_epub()

		return epub_file_path

	def __str__(self):
		str = "Stash content:\n"
		for k, l in self.lists.iteritems():
			str = str + "\t" + k + ":\n"
			for book in l:
				str = str + "\t\t" + book.__str__() + "\n"

		return str





#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------


import logging

log = logging.getLogger(__name__)

class Metadata(object):
	def __init__(self):
		self.first_title = "Untitled"
		self.meta_list = []

	def validate_metadata(self, name, role, file_as, event, scheme):
		from attributes import metadata_attributes, metadata_roles
		
		if name not in metadata_attributes.iterkeys():
			log.warning("Invalid metadata name: %s" % name)
			return False

		if role != "":
			if "role" not in metadata_attributes[name]:
				log.warning("Invalid metadata attribute: role")
				return False
			if role not in metadata_roles:
				log.warning("Invalid role attribute: %s" % role)
				return False
			
		if (file_as != "") and ("file-as" not in metadata_attributes[name]):
			log.warning("Invalid metadata attribute: file-as")
			return False

		if (event != "") and ("event" not in metadata_attributes[name]):
			log.warning("Invalid metadata attribute: event")
			return False

		if (scheme != "") and ("scheme" not in metadata_attributes[name]):
			log.warning("Invalid metadata attribute: scheme")
			return False
		
		log.debug("Metadata validation passed!")
		return True

	def extract_from_csv(self, _file):
		import csv
		
		log.debug("Parsing the CSV file...")
		with open(_file, 'rb') as f:
			reader = csv.reader(f)

			first_title_found = False
			for row in reader:
				log.debug(row)
				name = row[0].decode("utf-8")
				value = row[1].decode("utf-8")
				role = row[2].decode("utf-8")
				file_as = row[3].decode("utf-8")
				event = row[4].decode("utf-8")
				scheme = row[5].decode("utf-8")

				if name == "name":
					"""The first row may include headers"""
					continue

				if (not first_title_found) and (name == "title"):
					self.first_title = value
					first_title_found = True
				else:
					if self.validate_metadata(name, role, file_as, event, scheme):
						self.meta_list.append((name, value, role, file_as, event, scheme))

		self.meta_list.append(("contributor", "Book Binder", "bkp", "", "", ""))


if __name__ == '__main__':
	logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
	logging.getLogger(__name__)
	
	Metadata().extract_from_csv("test-data/metadata.csv")



#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------


from bs4 import BeautifulSoup
import cssutils
import logging

from templates import content_template

log = logging.getLogger(__name__)

class Sanitizer:
	def sanitize(self, content):
		soup = BeautifulSoup(content)
		css = self.extract_css(soup)
		alltags, usables = self.find_usable_rules(css)
		clean_soup = self.clean(soup, alltags, usables)

		ret_soup = BeautifulSoup(content_template)
		ret_soup.body.replace_with(clean_soup.body)

		return unicode(ret_soup)

	def extract_css(self, soup):
		css_sheet = cssutils.css.CSSStyleSheet()

		for styletag in soup.find_all('style', type='text/css'):
			if not styletag.string: # probably an external sheet
				continue
			for rule in cssutils.parseString(styletag.string):
				css_sheet.add(rule)
		
		log.debug("Extracted embedded stylesheet:\n%s" % css_sheet.cssText)
		return css_sheet

	def find_usable_rules(self, sheet):
		"""Finds those CSS classes which will be used in the final HTMl file.
		These include the styles listed in the "usables" dictionary.
		"""
		usables = {
			"bolds": [],
			"italics": [],
			"blockquotes": [],
			"double_blockquotes": [],
			"triple_blockquotes": [],
			"underlines": [],
			"right_aligned": [],
			"centered": [],
			"strikethrus": [],
			"subscripts": [],
			"superscripts": [],
			"alltags": [],
		}

		alltags = []
		
		for rule in sheet:
			if (rule.type == rule.STYLE_RULE) and (rule.selectorText.startswith(".c")):
				alltags.append(rule.selectorText[1:])
				for property in rule.style:
					if property.name == "font-weight" and property.value == "bold":
							usables["bolds"].append(rule.selectorText[1:])
						
					if property.name == "font-style" and property.value == "italic":
							usables["italics"].append(rule.selectorText[1:])
							
					if property.name == "margin-left":
						if property.value == "36pt":
							usables["blockquotes"].append(rule.selectorText[1:])
						if property.value == "72pt":
							usables["double_blockquotes"].append(rule.selectorText[1:])
						if property.value == "108pt":
							usables["double_blockquotes"].append(rule.selectorText[1:])
				
					if property.name == "text-align":
						if property.value == "center":
							usables["centered"].append(rule.selectorText[1:])
						if property.value == "right":
							usables["right_aligned"].append(rule.selectorText[1:])
							
					if property.name == "text-decoration":
						if property.value == "line-through":
							usables["strikethrus"].append(rule.selectorText[1:])
						if property.value == "underline":
							usables["underlines"].append(rule.selectorText[1:])

					if property.name == "vertical-align":
						if property.value == "super":
							usables["superscripts"].append(rule.selectorText[1:])
						if property.value == "sub":
							usables["subscripts"].append(rule.selectorText[1:])

		return (alltags, usables)

	def remove_class_from_tag(self, tag, cl):
		tag.get('class').remove(cl)
		if (len(tag.get('class'))) < 1:
			del tag["class"]

		return tag


	def tagify(self, class_list, soup, ntag):
		body = soup.body
		for cl in class_list:
			for tag in body.find_all(class_=cl):
				tag.wrap(soup.new_tag(ntag))
				tag = self.remove_class_from_tag(tag, cl)
		
		
	def remove_unused_classes(self, soup, alltags, usables):
		body = soup.body

		log.debug("Removing all class attributes except for the <span> and <p> tags.")
		for tag in soup.find_all():
			if tag.name.lower() not in ["span", "p"]:
				if tag.has_attr('class'):
					del tag["class"]

		all_usables = []
		for v in usables.itervalues():
			all_usables = all_usables + v

		for class_tag in alltags:
			if class_tag not in all_usables:
				for tag in body.find_all(class_=class_tag):
					tag = self.remove_class_from_tag(tag, class_tag)

		return soup
				
	def clean(self, in_soup, alltags, usables):
		soup = self.remove_unused_classes(in_soup, alltags, usables)

		self.tagify(usables["bolds"], soup, "strong")
		self.tagify(usables["italics"], soup, "em")
		self.tagify(usables["subscripts"], soup, "sub")
		self.tagify(usables["superscripts"], soup, "sup")
		self.tagify(usables["blockquotes"], soup, "blockquote")

		body = soup.body
		del body["class"]

		for cl in usables["double_blockquotes"]:
			for tag in body.find_all(class_=cl):
				tag.wrap(soup.new_tag("blockquote"))
				tag.wrap(soup.new_tag("blockquote"))
				tag = self.remove_class_from_tag(tag, cl)
		#TODO: This doesn't seem to work
		for cl in usables["triple_blockquotes"]:
			for tag in body.find_all(class_=cl):
				tag.wrap(soup.new_tag("blockquote"))
				tag.wrap(soup.new_tag("blockquote"))
				tag.wrap(soup.new_tag("blockquote"))
				tag = self.remove_class_from_tag(tag, cl)

		for cl in usables["underlines"]:
			for tag in body.find_all(class_=cl):
				tag["class"] = "underlined"

		for cl in usables["right_aligned"]:
			for tag in body.find_all(class_=cl):
				tag["class"] = "right-aligned"

		for cl in usables["centered"]:
			for tag in body.find_all(class_=cl):
				tag["class"] = "center-aligned"

		for cl in usables["strikethrus"]:
			for tag in body.find_all(class_=cl):
				tag["class"] = "strike-through"

		for tag in body.find_all("span"):
			if tag.get('class') == None:
				tag.unwrap()

		# Used for the footnotes
		"""
		for tag in body.find_all("div"):
			if tag.get('class') == None:
				tag.unwrap()
		"""
		
		for tag in body.find_all("a"):
			if tag.text == "":
				tag.unwrap()

		return soup

def wrap_content(content):
	soup = BeautifulSoup(content)

	ret_soup = BeautifulSoup(content_template)
	ret_soup.body.append(soup)

	return unicode(ret_soup)

if __name__ == '__main__':
	with open("test-data/TestDoc.html") as f:
		sanitizer = Sanitizer()
		content = f.read()
		content = sanitizer.sanitize(content)

		with open("outfile.html", "w") as f:
			f.write(content.encode('utf8'))



metadata_attributes = {
		"title": [],
		"creator": ["role", "file-as"],
		"subject": [],
		"description": [],
		"publisher": [],
		"contributor": ["role", "file-as"],
		"date": ["event"],
		"type": [],
		"format": [],
		"source": [],
		"language": [],
		"relation": [],
		"coverage": [],
		"identifier": ["scheme"],
		"rights": []
}

metadata_roles = [
	"abr",
	"acp",
	"act",
	"adi",
	"adp",
	"aft",
	"anl",
	"anm",
	"ann",
	"ant",
	"ape",
	"apl",
	"app",
	"aqt",
	"arc",
	"ard",
	"arr",
	"arr",
	"art",
	"asg",
	"asn",
	"ato",
	"att",
	"auc",
	"aud",
	"aui",
	"aus",
	"aut",
	"bdd",
	"bjd",
	"bkd",
	"bkd",
	"bkp",
	"blw",
	"bnd",
	"bpd",
	"brd",
	"brl",
	"bsl",
	"cas",
	"ccp",
	"chr",
	"cli",
	"cll",
	"clr",
	"clt",
	"cmm",
	"cmp",
	"cmt",
	"cnd",
	"cng",
	"cns",
	"coe",
	"col",
	"com",
	"con",
	"cor",
	"cos",
	"cot",
	"cou",
	"cov",
	"cpc",
	"cpe",
	"cph",
	"cpl",
	"cpt",
	"cre",
	"crp",
	"crr",
	"crr",
	"crt",
	"csl",
	"csp",
	"cst",
	"ctb",
	"cte",
	"ctg",
	"ctr",
	"cts",
	"ctt",
	"cur",
	"cwt",
	"dbp",
	"dfd",
	"dfe",
	"dft",
	"dgg",
	"dgs",
	"dis",
	"dln",
	"dnc",
	"dnr",
	"dpc",
	"dpt",
	"drm",
	"drt",
	"dsr",
	"dst",
	"dtc",
	"dte",
	"dtm",
	"dto",
	"dub",
	"edc",
	"edm",
	"edt",
	"egr",
	"elg",
	"elt",
	"eng",
	"enj",
	"etr",
	"evp",
	"exp",
	"fac",
	"fac",
	"fds",
	"fld",
	"flm",
	"fmd",
	"fmk",
	"fmo",
	"fmo",
	"fmp",
	"fnd",
	"fpy",
	"frg",
	"gis",
	"his",
	"hnr",
	"hst",
	"ill",
	"ill",
	"ilu",
	"ins",
	"ins",
	"inv",
	"isb",
	"itr",
	"ive",
	"ivr",
	"jud",
	"jug",
	"lbr",
	"lbt",
	"ldr",
	"led",
	"lee",
	"lel",
	"len",
	"let",
	"lgd",
	"lie",
	"lil",
	"lit",
	"lsa",
	"lse",
	"lso",
	"ltg",
	"lyr",
	"mcp",
	"mdc",
	"med",
	"mfp",
	"mfr",
	"mfr",
	"mod",
	"mon",
	"mrb",
	"mrk",
	"msd",
	"mte",
	"mtk",
	"mus",
	"nrt",
	"opn",
	"org",
	"orm",
	"osp",
	"oth",
	"own",
	"pan",
	"pat",
	"pbd",
	"pbl",
	"pdr",
	"pfr",
	"pht",
	"plt",
	"pma",
	"pmn",
	"pop",
	"ppm",
	"ppt",
	"pra",
	"prc",
	"prd",
	"pre",
	"prf",
	"prf",
	"prg",
	"prm",
	"prn",
	"pro",
	"prp",
	"prs",
	"prt",
	"prv",
	"pta",
	"pte",
	"ptf",
	"pth",
	"ptt",
	"pup",
	"rbr",
	"rcd",
	"rce",
	"rcp",
	"rdd",
	"red",
	"ren",
	"res",
	"rev",
	"rpc",
	"rps",
	"rpt",
	"rpy",
	"rse",
	"rsg",
	"rsp",
	"rsr",
	"rst",
	"rth",
	"rtm",
	"sad",
	"sce",
	"scl",
	"scr",
	"sds",
	"sec",
	"sgd",
	"sgn",
	"sht",
	"sll",
	"sng",
	"spk",
	"spn",
	"spy",
	"srv",
	"std",
	"stg",
	"stl",
	"stm",
	"stn",
	"str",
	"tcd",
	"tch",
	"ths",
	"tld",
	"tlp",
	"trc",
	"trl",
	"tyd",
	"tyg",
	"uvp",
	"vac",
	"vdg",
	"wac",
	"wal",
	"wam",
	"wat",
	"wdc",
	"wde",
	"win",
	"wit",
	"wpr",
	"wst"
]





#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------


content_template = """<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">

<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title></title>
</head>
<body>
</body>
</html>"""

toc_ncx = u"""<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE ncx PUBLIC "-//NISO//DTD ncx 2005-1//EN"
 "http://www.daisy.org/z3986/2005/ncx-2005-1.dtd">
<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1">
  <head>
    <meta name="dtb:uid" content="{book.uuid}" />
    <meta name="dtb:depth" content="{book.toc_root.maxlevel}" />
    <meta name="dtb:totalPageCount" content="0" />
    <meta name="dtb:maxPageNumber" content="0" />
  </head>
  <docTitle>
    <text>{book.title}</text>
  </docTitle>
  {navmap}
</ncx>"""

container_xml = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<container xmlns="urn:oasis:names:tc:opendocument:xmlns:container" version="1.0">
  <rootfiles>
    <rootfile full-path="OEBPS/content.opf" media-type="application/oebps-package+xml"/>
  </rootfiles>
</container>
"""




#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------

from setuptools import setup
import re

version = "0.2.1"
description = "ePub generation from Google Documents"
url = "https://github.com/canpolat/bookbinder"
author = "Eren Inan Canpolat"
author_email = "eren.canpolat@gmail.com"

setup(name='bookbinder',
	version=version,
	description=description,
	long_description=open('README.rst').read(),
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Environment :: Console',
		'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
		'Natural Language :: English',
		'Programming Language :: Python :: 2.7',
	],
	url=url,
	author=author,
	author_email=author_email,
	packages=['bookbinder'],
	entry_points={
		'console_scripts': [
			'bookbinder = bookbinder:bindbook',
		],
	},
	install_requires=[
		"Genshi>=0.6",
		"Markdown>=2.4",
		"beautifulsoup4>=4.3.2",
		"cssutils>=1.0",
		"google-api-python-client>=1.2",
	],
	include_package_data=True,
	zip_safe=False)



#!/usr/bin/python
# -*- coding: utf-8 -*-

#--------------------------------------------------------------------
# Copyright (c) 2014 Eren Inan Canpolat
# Author: Eren Inan Canpolat <eren.canpolat@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--------------------------------------------------------------------

import sys
import tempfile
import logging

import gdrive
import stash

log = logging.getLogger("bookbinder.main")

import pkg_resources
version = pkg_resources.require("bookbinder")[0].version

def init_logging():
	logging.basicConfig(level=logging.DEBUG,
				format='%(asctime)s %(name)-20s %(levelname)-8s %(message)s',
				datefmt='%m-%d %H:%M',
				filename='bookbinder.log',
				filemode='w')

	console = logging.StreamHandler()
	console.setLevel(logging.INFO)
	formatter = logging.Formatter('%(name)-24s: %(levelname)-8s %(message)s')
	console.setFormatter(formatter)
	logging.getLogger('').addHandler(console)

	apiclient_log = logging.getLogger("apiclient")
	apiclient_log.setLevel(logging.WARNING)

	oauth2client_log = logging.getLogger("oauth2client")
	oauth2client_log.setLevel(logging.WARNING)


def dry_run(folders, cache=False):
	drive = gdrive.GDrive(cache)

	log.info("================================")
	log.info("=============DRY RUN============")
	log.info("================================")

	for folderId in folders:
		if folderId == None:
			continue
		log.info("Processing folder: %s" % str(folderId))
		file_ids = drive.get_file_list(folderId)

		log.info("Number of files in the folder: %s" % str(len(file_ids)))

		count = 1
		for fid in file_ids:
			log.info("{:02d} - File ID: {}".format(count, fid))
			drive.print_file_metadata(log, fid, prefix="\t\t")
			count = count + 1

	log.info("=======DRY RUN COMPLETE=========")
	log.info("================================")

def process(folders, cache=False):
	drive = gdrive.GDrive(cache)
	dirpath = tempfile.mkdtemp()
	if len(folders) > 0:
		stash_ = stash.Stash(dirpath, folders[0])
	total_count = 0

	for folderId in folders:
		log.info("Processing folder: %s" % str(folderId))
		file_ids = drive.get_file_list(folderId)
		log.info("Number of files in the folder: %s" % str(len(file_ids)))

		if len(file_ids) > 0:
			total_count = total_count + len(file_ids)

			count = 0
			for fid in file_ids:
				log.debug("File ID: %s" % fid)
				log.debug("Retrieving file metadata...")
				dfile = drive.get_file_metadata(fid)
				processor = gdrive.GDriveFileProcessorFactory().get_processor(dfile)
				log.debug("Processing file...")
				bfile = processor.process(drive.service, dfile)
				bfile.enqueue(stash_)
				count = count + 1
			log.info("Number of files downloaded: %s" % count)

	if total_count > 0:
		epub_path = stash_.export()
		log.info("ePub ready at: %s" % epub_path)
	else:
		log.error("No files found. Please check the folder ID(s) you provided. If you don't know what a folder ID is, please see the user guide.")

def bindbook():
	import argparse

	init_logging()

	description= "ePub generation from Google Documents."
	parser = argparse.ArgumentParser(description=description)

	parser.add_argument("folders", nargs="+", help="ID(s) of the Google Drive folder(s) containing book content")
	parser.add_argument("-c", "--cache", action="store_true", help="cache credentials and use them for subsequent file access")
	parser.add_argument("-d", "--dryrun", action="store_true", help="list the files that would be processed for the book, but don't process them")
	parser.add_argument("-v", "--version", action="version", version="Book Binder version: {}".format(version))
	args = parser.parse_args()

	if args.dryrun:
		dry_run(args.folders, args.cache)
	else:
		process(args.folders, args.cache)


if __name__ == '__main__':
	bindbook()
